package domowe.r4.odmiana_slow;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class BazaOdmian {
	private static final String DOMYSLNY_PLIK = "PoliMorf-0.6.7.tab";
	
	// dla formy podstawowej zawiera zbiór odmian
	private final Map<String, Set<String>> odmiany = new HashMap<>();

	// dla formy odmienionej zawiera zbiór form podstawowych
	private final Map<String, Set<String>> podstawowe = new HashMap<>();
	
	// tylko po to, aby konstruktor nie był publiczny:
	private BazaOdmian() {
		
	}
	
	// oficjalnym, publicznym sposobem tworzenia obiektów jest ta metoda statyczna:
	public static BazaOdmian zPliku(String sciezka) {
		BazaOdmian baza = new BazaOdmian();
		baza.wczytajDaneZPliku(sciezka);
		return baza;
	}
	
	public static BazaOdmian zDomyslnegoPliku() {
		return BazaOdmian.zPliku(DOMYSLNY_PLIK);
	}
	
	// to jest metoda pomocnicza, więc oznaczam jako prywatną
	private void wczytajDaneZPliku(String sciezka) {
		try(BufferedReader input = new BufferedReader(new FileReader(sciezka))) {
			String linia;
			while((linia = input.readLine()) != null) {
				String[] t = linia.split("\t", 3);				
//				final String formaPodstawowa = t[1];
//				final String formaOdmieniona = t[0];

				// intern znajduje "wewnętrznego" stringa o takiej samej treści
				// co pozwala zaoszczędzić pamięć, ale spowalnia wczytywanie
				final String formaPodstawowa = t[1].intern();
				final String formaOdmieniona = t[0].intern();
				
				Set<String> zbiorOdmian = odmiany.get(formaPodstawowa);
				if(zbiorOdmian == null) {
					zbiorOdmian = new HashSet<>();
					odmiany.put(formaPodstawowa, zbiorOdmian);
				}
				zbiorOdmian.add(formaOdmieniona);

				Set<String> zbiorPodstawowych = podstawowe.get(formaOdmieniona);
				if(zbiorPodstawowych == null) {
					zbiorPodstawowych = new HashSet<>();
					podstawowe.put(formaOdmieniona, zbiorPodstawowych);
				}
				zbiorPodstawowych.add(formaPodstawowa);
			}
			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public Set<String> znajdzFormyPodstawowe(String slowoOdmienione) {
		Set<String> wyniki = podstawowe.get(slowoOdmienione);
		if(wyniki == null) {
			return Collections.emptySet(); // od Javy 9 można też Set.of()
		} else {
			// zwracam zbiór niemodyfikowalny, aby klient tej klasy nie miał prawa modyfikowania zawartości bazy odmian
			return Collections.unmodifiableSet(wyniki);
		}
	}

	public Set<String> znajdzOdmiany(String slowo) {
		Set<String> wyniki = odmiany.get(slowo);
		if(wyniki == null) {
			return Collections.emptySet();
		} else {
			return Collections.unmodifiableSet(wyniki);
		}
	}

}
